/**
	Cave Battle
	Dark caves with narrow connections set the stage for this battle. Every player
	only has a single clonk and no relaunches, so caution is needed.

	@author Maikel
*/


// List for storing the different large caves.
static cave_list;

protected func Initialize()
{
	// Rescale cave coordinates with map zoom and shuffle them.
	var mapzoom = GetScenarioVal("MapZoom", "Landscape");
	for (var cave in cave_list)
	{
		cave[0] *= mapzoom;
		cave[1] *= mapzoom;	
	}
	ShuffleArray(cave_list);
	
	// Initialize different parts of the scenario.
	InitVegetation();
	InitMaterials();
	InitEnvironment();
	return;
}

protected func InitializePlayer(int plr)
{
	// Get the only clonk of the player.
	var clonk = GetCrew(plr);
	
	// Players start in a random small cave.
	var cave = cave_list[BoundBy(plr, 0, GetLength(cave_list) - 1)];
	clonk->SetPosition(cave[0], cave[1]);

	// Players start with a shovel, a pickaxe and two firestones.
	clonk->CreateContents(Shovel);
	clonk->CreateContents(Pickaxe);
	clonk->CreateContents(Torch);
	clonk->CreateContents(Firestone, 2);
	
	// Set the zoom range to be standard low, but allow for zooming out
	// such that light sources a bit further away can be spotted.
	SetPlayerZoomByViewRange(plr, 300, nil, PLRZOOM_Direct);
	SetPlayerZoomByViewRange(plr, 600, nil, PLRZOOM_LimitMax);
	SetPlayerViewLock(plr, true);
	return;
}

/*-- Scenario Initiliaztion --*/

private func InitVegetation()
{
	// Cave mushrooms provide wood, extra place them in the large caves.
	LargeCaveMushroom->Place(100 + Random(30), nil, { terrafom = false });
	
	// Some mushrooms to regain health.
	Mushroom->Place(80);
	Fern->Place(60);
	
	// Some objects in the earth.	
	PlaceObjects(Loam, 50 + Random(30), "Earth");
	PlaceObjects(Firestone, 50 + Random(30), "Earth");
	PlaceObjects(Dynamite, 20 + Random(10), "Earth");
	PlaceObjects(DynamiteBox, 10 + Random(5), "Rock");
	
	return;
}

private func InitMaterials()
{
	var wdt = LandscapeWidth();
	var hgt = LandscapeHeight();
	
	// Create lorries at random small caves, but only for half of them.
	for (var index = GetLength(cave_list) - 1; index >= GetLength(cave_list) / 2; index--)
	{
		var cave = cave_list[index];
		var lorry = CreateObject(Lorry, cave[0], cave[1]);
		// Basic objects which are in every lorry.
		lorry->CreateContents(Dynamite, RandomX(2, 4));
		lorry->CreateContents(Loam, RandomX(2, 4));
		// Objects which are only in half of the lorries.
		if (!Random(2)) 
			lorry->CreateContents(DynamiteBox, RandomX(1, 2));
		if (!Random(2)) 
			lorry->CreateContents(Shield);
		if (!Random(2)) 
			lorry->CreateContents(Torch, 2);
		// Objects which are only in one third of the lorries.
		if (!Random(3)) 
			lorry->CreateContents(GrappleBow, RandomX(1, 2));
		if (!Random(3)) 
			lorry->CreateContents(Bread, RandomX(1, 2));
		if (!Random(3))
		{
			lorry->CreateContents(Bow);
			lorry->CreateContents([Arrow, FireArrow, FireArrow][Random(3)], 2);
		}
		// Objects which are only in one fifth of the lorries.
		if (!Random(5)) 
			lorry->CreateContents(Javelin, RandomX(1, 2));
		if (!Random(5)) 
			lorry->CreateContents(Club, RandomX(1, 2));
		// Objects which are only in one eighth of the lorries.
		if (!Random(8))
			lorry->CreateContents(IronBomb, RandomX(1, 2));
		if (!Random(8))
			lorry->CreateContents(WallKit, 1);
		if (!Random(8))
		{
			lorry->CreateContents(Musket);
			lorry->CreateContents(LeadShot);
		}
	}
	
	// Create two lorries at the main cave.
	for (var side = -1; side <= 1; side += 2)
	{
		// Both sides of the cave are lighted for all players.
		CreateLight(wdt / 2 + side * 60, hgt / 2 + 20, 120, Fx_Light.LGT_Constant);
		// Create lorry with useful tools and weapons.
		var lorry = CreateObject(Lorry, wdt / 2 + side * 60, hgt / 2 + 40);
		lorry->CreateContents(Bow, 2);
		lorry->CreateContents(BombArrow, 4);
		lorry->CreateContents(Boompack, 2);
		lorry->CreateContents(PowderKeg, 2);
		lorry->CreateContents(TeleGlove, 1);
		lorry->CreateContents(WindBag, 1);
	}

	return;
}

private func InitEnvironment()
{

	return;
}


